#include "receiver.h"

uint8_t receiver::prev; // remembers state of input bits from previous interrupt
uint32_t receiver::risingEdge[2]; // time of last rising edge for each channel
volatile uint32_t receiver::uSec[2]; // the latest measured pulse width in uS for each channel

void receiver::start(void) {
  for (int pin = 0; pin <= 1; pin++) { // enable PB0 and PB1 (physical pins 5 and 6 ) as our 2 input bits
    pinMode(pin, INPUT_PULLUP);
  }
  for (int n = 0; n < 2; n++) {
    risingEdge[n] = 0x80000000UL; // default rising edge times so that active() returns false before a signal is received
  }

  PCMSK |= 0x03; // set the mask to allow those 2 pins to generate interrupts
  GIMSK |= 0x20;  // enable pinchange interupts
}

uint16_t receiver::channel(uint8_t n) { // return uS for channel[n]. n is channel 0 or 1.
   uint16_t uS;
   cli();
   uS = (uint16_t)uSec[n];
   sei();
   if (uS < 1000)
    uS = 1000;
   else if (uS > 2000)
    uS = 2000;
   return uS;
}

bool receiver::active(uint8_t n) { // returns true if channel[n] has received +ve edge in last half-second
  uint32_t uS = micros();
  cli();
  uS -= risingEdge[n];
  sei();
  return uS < 500000UL;
}

ISR(PCINT0_vect) { // one or more of pins 5 or 6 have changed state
  uint32_t now = micros();
  uint8_t curr = PINB & 0x03; // current state of the 2 input bits
  uint8_t changed = curr ^ receiver::prev;
  uint8_t mask = 0x01;
  for (uint8_t channel = 0; channel < 2;  channel++) {
    if (changed & mask) { // this pin has changed state
      if (curr & mask) { // +ve edge so remember time
        receiver::risingEdge[channel] = now;
      }
      else { // -ve edge so store pulse width
        receiver::uSec[channel] = now - receiver::risingEdge[channel];
      }
    }
    mask <<= 1;
  }
  receiver::prev = curr;
}
